/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <qjson/parser.h>

#include <file/CBasicSocket.h>

CBasicSocket::CBasicSocket(QObject * parent)
    : QTcpSocket(parent), m_expected_size(0)
{
    connect(this, SIGNAL(disconnected()), SLOT(deleteLater()));
    connect(this, SIGNAL(error(QAbstractSocket::SocketError)), SLOT(OnError(QAbstractSocket::SocketError)));
    connect(this, SIGNAL(readyRead()),    SLOT(OnReadyRead()));
}

QVariantMap CBasicSocket::ParseJSON(QByteArray json)
{
    return QJson::Parser().parse(json).toMap();
}

void CBasicSocket::SendData(QByteArray data)
{
    // Convert the size of the data into a string, terminate
    // it with EOT, and then send the raw data over the socket.
    write(QByteArray::number(data.size()));
    write("\x04");
    write(data);
}

void CBasicSocket::SendStatus(QString status)
{
    SendData(tr("{\"status\": \"%1\"}").arg(status).toUtf8());
}

void CBasicSocket::OnError(QAbstractSocket::SocketError)
{
    // If the socket is open, disconnect - otherwise delete it
    if(state() == QAbstractSocket::ConnectedState)
        disconnectFromHost();
    else
        deleteLater();
}

void CBasicSocket::OnReadyRead()
{
    // Append whatever is pending to the read buffer
    m_read_buffer += readAll();

    // If we don't know the size, then check for our control character in the stream
    if(!m_expected_size && m_read_buffer.contains('\x04'))
    {
        int offset = m_read_buffer.indexOf('\x04');
        m_expected_size = m_read_buffer.left(offset).toLongLong();

        // Remove the size off the front
        m_read_buffer.remove(0, offset + 1);
    }

    // If we do know the size, then check to see if we're done
    if(m_expected_size && m_read_buffer.size() >= m_expected_size)
    {
        emit Data(m_read_buffer);

        // Clear everything for the next transfer
        m_read_buffer.remove(0, m_expected_size);
        m_expected_size = 0;
    }
}
