/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <QCoreApplication>
#include <QFileDialog>
#include <QIcon>
#include <QStringList>

#include <dialogs/CAboutDialog.h>
#include <dialogs/CSettingsDialog.h>
#include <file/CFileSender.h>
#include <main/CTrayIcon.h>

CTrayIcon::CTrayIcon()
    : m_pc_menu("Send Files To")
{
    // Connect the signals for the broadcast and file servers
    connect(&m_broadcast_server, SIGNAL(Message(QString,QSystemTrayIcon::MessageIcon)),
                                 SLOT(OnMessage(QString,QSystemTrayIcon::MessageIcon)));
    connect(&m_file_server, SIGNAL(Message(QString,QSystemTrayIcon::MessageIcon)),
                            SLOT(OnMessage(QString,QSystemTrayIcon::MessageIcon)));

    // Initialize the two servers
    m_broadcast_server.Init();
    m_file_server.Init();

    connect(&m_pc_menu, SIGNAL(aboutToShow()), SLOT(OnShowMachineList()));

    // Initialize the icon
    setIcon(QIcon(":/icons/light.png"));
    CreateContextMenu();

    // Display the icon
    setVisible(true);
}

void CTrayIcon::OnAbout()
{
    CAboutDialog().exec();
}

void CTrayIcon::OnMessage(QString message, QSystemTrayIcon::MessageIcon icon)
{
    showMessage("NitroShare Notification", message, icon);
}

void CTrayIcon::OnSendFile()
{
    // Ask the user for the files they wish to send
    QStringList filenames = QFileDialog::getOpenFileNames(NULL, "Select Files to Send");
    if(filenames.size())
    {
        QAction * action = qobject_cast<QAction *>(sender());
        QByteArray host = action->text().toUtf8();

        if(m_broadcast_server.GetMachineList().contains(host))
        {
            CFileSender * client = new CFileSender(this, host, filenames);
            connect(client, SIGNAL(Message(QString,QSystemTrayIcon::MessageIcon)),
                            SLOT(OnMessage(QString,QSystemTrayIcon::MessageIcon)));
            client->Start(m_broadcast_server.GetMachineList()[host]);
        }
        // TODO: throw error here
        else {}
    }
}

void CTrayIcon::OnSettings()
{
    // Display the settings dialog...
    CSettingsDialog().exec();

    // ...and then instruct each of the other classes to
    // begin reloading settings.
    m_broadcast_server.Init();
    m_file_server.Init();
}

void CTrayIcon::OnShowMachineList()
{
    // Clear the list and add all current machines
    m_pc_menu.clear();

    foreach(CMachine machine, m_broadcast_server.GetMachineList())
        m_pc_menu.addAction(machine.name, this, SLOT(OnSendFile()));
}

void CTrayIcon::CreateContextMenu()
{
    QMenu * menu = new QMenu;

    menu->addMenu(&m_pc_menu);
    menu->addSeparator();
    menu->addAction("Settings", this, SLOT(OnSettings()));
    menu->addAction("About",    this, SLOT(OnAbout()));
    menu->addSeparator();
    menu->addAction("Exit", QCoreApplication::instance(), SLOT(quit()));

    setContextMenu(menu);
}
